-----------------------
-------VARIABLES-------
-----------------------


local ADDON_NAME, namespace = ...;
local variables = namespace.variables;
local functions = namespace.functions;


-----------------------
-----SLASH FUNCTION----
-----------------------


SLASH_LLR1 = "/llr";
SLASH_LLR2 = "/lptlootroll";
SLASH_LLR3 = "/lpt";

SlashCmdList.LLR = 
function(msg)
	msg = strlower(msg);

	if variables.commandArray[msg] then
		variables.commandArray[msg].func();
		return;
	end
	
	print("");
	functions.printOut(GetAddOnMetadata("LPT Loot Roll", "Title") .. " version: " .. GetAddOnMetadata("LPT Loot Roll", "Version"));
	
	if msg and msg ~= "" then
		functions.printOut("Error: Command not recognized.");
	else
		functions.printOut("Error: No command recieved.");
	end

	functions.printCommands();
end


-----------------------
----FILTER FUNCTION----
-----------------------


--Filter function to replace the normal string with a link that promts the clicker if they wish to change their roll settings to the values of the sender.
--As seen in the weakaura addon (https://github.com/WeakAuras/WeakAuras2/blob/master/WeakAuras/Transmission.lua), simplified for my usage.
local function filterFunction(_, _, msg, ...)
	local _, _, name, rollValues = msg:find("%[LPTLootRoll: ([^%s]+) %- ([^%]]+)%]");

	if name and rollValues then
		msg = "|Hgarrmission:lptlootroll:rollvalues&" .. rollValues .. "|h|cFF00CCFF[LPT Roll Settings: " .. name .. "]|h|r";
		return false, msg, ...;
	end
end

ChatFrame_AddMessageEventFilter("CHAT_MSG_SAY", filterFunction);
ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER", filterFunction);
ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER_INFORM", filterFunction);
ChatFrame_AddMessageEventFilter("CHAT_MSG_CHANNEL", filterFunction);
ChatFrame_AddMessageEventFilter("CHAT_MSG_YELL", filterFunction);
ChatFrame_AddMessageEventFilter("CHAT_MSG_PARTY", filterFunction);
ChatFrame_AddMessageEventFilter("CHAT_MSG_PARTY_LEADER", filterFunction);
ChatFrame_AddMessageEventFilter("CHAT_MSG_RAID", filterFunction);
ChatFrame_AddMessageEventFilter("CHAT_MSG_RAID_LEADER", filterFunction);
ChatFrame_AddMessageEventFilter("CHAT_MSG_INSTANCE_CHAT", filterFunction);
ChatFrame_AddMessageEventFilter("CHAT_MSG_INSTANCE_CHAT_LEADER", filterFunction);
ChatFrame_AddMessageEventFilter("CHAT_MSG_GUILD", filterFunction);
ChatFrame_AddMessageEventFilter("CHAT_MSG_OFFICER", filterFunction);
ChatFrame_AddMessageEventFilter("CHAT_MSG_BN_WHISPER", filterFunction);
ChatFrame_AddMessageEventFilter("CHAT_MSG_BN_WHISPER_INFORM", filterFunction);

--Get data from the custom hyperlinks.
hooksecurefunc("SetItemRef", 
	function(link, text)
		local linkType, addon, param1 = strsplit(":", link);
		
		if linkType == "garrmission" and addon == "lptlootroll" then
			local func, value = strsplit("&", param1);

			if func and func ~= "" and value and value ~= ""  then
				if func == "rollvalues" then 
					local dialog = StaticPopup_Show("LPTLootRoll_SettingShare");

					if dialog then
						dialog.data = value;
					end
				elseif func == "command" then
					if variables.commandArray[value] then
						variables.commandArray[value].func();
					end
				elseif func == "roller" then
					if not variables.rollHistory[1].distributed then 
						local dialog = StaticPopup_Show("LPTLootRoll_DistributeItem", value);
						
						if dialog then
							dialog.data = value;
						end
					end
				end
			end
		end
	end
);


-----------------------
----GLOBAL FUNCTIONS---
-----------------------


--Function for printing out all available commands
function functions.printCommands()
	print("");
	functions.printOut("Available commands are:");
	
	for i, v in pairs(variables.commandArray) do
		print("|Hgarrmission:lptlootroll:command&" .. i .. "|h|cFF00CCFF[" ..i:gsub("^%l", strupper) .. "]|h|r|cFFFFFF00 - " .. v.description);
	end
end

--Custom printout method for yellow message printouts.
function functions.printOut(msg)
	DEFAULT_CHAT_FRAME:AddMessage(msg,1,1,0);
end

--Input the current item into the roll frame.
function functions.setItemButtonData(frame, link)
	frame:SetScript("OnMouseDown", 
		function(self,button) 
			if IsModifiedClick("CHATLINK") then 
				ChatEdit_InsertLink(link);
			elseif IsModifiedClick("DRESSUP") then 
				DressUpItemLink(link);
			elseif IsShiftKeyDown() and button=="RightButton" then 
				OpenAzeriteEmpoweredItemUIFromLink(link) 
			end 
		end
	);
	
	frame:SetNormalTexture(select(10,GetItemInfo(link)));
	
	frame:SetScript("OnEnter", 
		function(self) 
			GameTooltip:SetOwner(self, "ANCHOR_CURSOR"); 
			GameTooltip:ClearLines(); 
			GameTooltip:SetHyperlink(link); 
			GameTooltip:Show();
		end
	);

	frame:SetScript("OnLeave", 
		function(self) 
			GameTooltip:Hide();
			ResetCursor();
		end
	);
end

--Function for controlling length of history array.
function functions.historyLengthControl()
	while #variables.rollHistory >= llrSettings.historyLength do
		tremove(variables.rollHistory);
	end
end

--Get raidleader
function functions.getRaidLeader()
	for i = 1, GetNumGroupMembers() do
		local name, rank = GetRaidRosterInfo(i);

		if name and rank == 2 then
			return name;
		end
	end
end

--Function for calculating scaling value. When scaling for screens with a higher Y resolution than 1200, you have to take parentscale into the account for it to scale properly.
function functions.calculateScale()
	local yResolution = tonumber(strmatch(GetCVar("gxWindowedResolution"), "%d+x(%d+)"));
	local parentScale = UIParent:GetScale();

	return 768/yResolution/(yResolution > 1200 and parentScale or 1);
end

--Function for showing the config window.
function functions.toggleConfig()
	InterfaceOptionsFrame_OpenToCategory(LPTLootRoll_Config);
	if not LPTLootRoll_Config:IsShown() then
		InterfaceOptionsFrame_OpenToCategory(LPTLootRoll_Config);
	end
end

--Function for hiding tooltip on mouseover.
function functions.hideGametoolTip()
	GameTooltip:Hide();
	ResetCursor();
end

--Function for getting the class color of a given player.
function functions.getClassColor(player)
	local _, playerClass = UnitClass(player);
    return RAID_CLASS_COLORS[playerClass];
end