-----------------------
-------VARIABLES-------
-----------------------


local ADDON_NAME, namespace = ...;
local variables = namespace.variables;
local functions = namespace.functions;

variables.prefixStatus = C_ChatInfo.RegisterAddonMessagePrefix("LPTLootRoll");
variables.rollHistory = {};
_, _, variables.playerClassId = UnitClass("player");
variables.playerName = UnitName("player");
variables.maxRollValue = 10000;

variables.commandArray = 
{
	["leader"] = 
	{
		func = 
			function() 
				functions.displayLeader()
			end,
		description = "Displays the raid leader window."
	},
	["user"] = 
	{
		func = 
			function() 
				functions.displayUser()
			end,
		description = "Displays the normal user window."
	},
	["item"] = 
	{
		func = 
			function() 
				functions.displayItemList()
			end,
		description = "Open the item list window."
	},
	["clear"] = 
	{
		func = 
			function() 
				StaticPopup_Show("LPTLootRoll_ClearHistory")
			end,
		description = "Clear all stored roll data."
	},
	["reset"] = 
	{
		func = 
			function() 
				StaticPopup_Show("LPTLootRoll_ResetPosition")
			end,
		description = "Resets position of windows."
	},
	["config"] = 
	{
		func = 
			function() 
				functions.toggleConfig()
			end,
		description = "Open the config menu."
	},
	["help"] = 
	{
		func = 
			function() 
				functions.printCommands();
			end,
		description = "Displays this menu."
	}
};

variables.classArray = 
{
	[1] = 				  -- Warrior
	{
		[4] = 			  -- Armor Class Type
		{
			[1] = false,  -- Cloth Armor 
			[2] = false,  -- Leather Armor
			[3] = false,  -- Mail Armor
			[4] = true,   -- Plate Armor
			[6] = true,   -- Shield Armor
		},
		[2] = 			  -- Weapons Class Type
		{
			[0]  = true,  -- One-Handed Axes
			[1]  = true,  -- Two-Handed Axes
			[2]  = false, -- Bows
			[3]  = false, -- Guns
			[4]  = true,  -- One-Handed Maces
			[5]  = true,  -- Two-Handed Maces
			[6]  = false, -- Polearms
			[7]  = true,  -- One-Handed Swords
			[8]  = true,  -- Two-Handed Swords
			[9]  = false, -- Warglaives
			[10] = false, -- Staves
			[13] = false, -- Fist Weapons
			[15] = false, -- Daggers
			[18] = false, -- Crossbows
			[19] = false, -- Wands
		},
		stats = {
			agility 	= false,
			strength 	= true,
			intellect 	= false
		},
	},
	[2] = 				  -- Paladin
	{
		[4] = 			  -- Armor Class Type
		{
			[1] = false,  -- Cloth Armor 
			[2] = false,  -- Leather Armor
			[3] = false,  -- Mail Armor
			[4] = true,   -- Plate Armor
			[6] = true,   -- Shield Armor
		},
		[2] = 			  -- Weapons Class Type
		{
			[0]  = true,  -- One-Handed Axes
			[1]  = true,  -- Two-Handed Axes
			[2]  = false, -- Bows
			[3]  = false, -- Guns
			[4]  = true,  -- One-Handed Maces
			[5]  = true,  -- Two-Handed Maces
			[6]  = false, -- Polearms
			[7]  = true,  -- One-Handed Swords
			[8]  = true,  -- Two-Handed Swords
			[9]  = false, -- Warglaives
			[10] = false, -- Staves
			[13] = false, -- Fist Weapons
			[15] = false, -- Daggers
			[18] = false, -- Crossbows
			[19] = false, -- Wands
		},
		stats = {
			agility 	= false,
			strength 	= true,
			intellect 	= true
		},
	},
	[3] = 				  -- Hunter
	{
		[4] = 			  -- Armor Class Type
		{
			[1] = false,  -- Cloth Armor 
			[2] = false,  -- Leather Armor
			[3] = true,   -- Mail Armor
			[4] = false,  -- Plate Armor
			[6] = false,  -- Shield Armor
		},
		[2] = 			  -- Weapons Class Type
		{
			[0]  = false, -- One-Handed Axes
			[1]  = false, -- Two-Handed Axes
			[2]  = true,  -- Bows
			[3]  = true,  -- Guns
			[4]  = false, -- One-Handed Maces
			[5]  = false, -- Two-Handed Maces
			[6]  = true,  -- Polearms
			[7]  = false, -- One-Handed Swords
			[8]  = false, -- Two-Handed Swords
			[9]  = false, -- Warglaives
			[10] = true,  -- Staves
			[13] = false, -- Fist Weapons
			[15] = false, -- Daggers
			[18] = true,  -- Crossbows
			[19] = false, -- Wands
		},
		stats = {
			agility 	= true,
			strength 	= false,
			intellect 	= false
		},
	},
	[4] = 				  -- Rogue
	{
		[4] = 			  -- Armor Class Type
		{
			[1] = false,  -- Cloth Armor 
			[2] = true,   -- Leather Armor
			[3] = false,  -- Mail Armor
			[4] = false,  -- Plate Armor
			[6] = false,  -- Shield Armor
		},
		[2] = 			  -- Weapons Class Type
		{
			[0]  = true,  -- One-Handed Axes
			[1]  = false, -- Two-Handed Axes
			[2]  = false, -- Bows
			[3]  = false, -- Guns
			[4]  = true,  -- One-Handed Maces
			[5]  = false, -- Two-Handed Maces
			[6]  = false, -- Polearms
			[7]  = true,  -- One-Handed Swords
			[8]  = false, -- Two-Handed Swords
			[9]  = false, -- Warglaives
			[10] = false, -- Staves
			[13] = true,  -- Fist Weapons
			[15] = true,  -- Daggers
			[18] = false, -- Crossbows
			[19] = false, -- Wands
		},
		stats = {
			agility 	= true,
			strength 	= false,
			intellect 	= false
		},
	},
	[5] = 				  -- Priest
	{
		[4] = 			  -- Armor Class Type
		{
			[1] = true,   -- Cloth Armor 
			[2] = false,  -- Leather Armor
			[3] = false,  -- Mail Armor
			[4] = false,  -- Plate Armor
			[6] = false,  -- Shield Armor
		},
		[2] = 			  -- Weapons Class Type
		{
			[0]  = false, -- One-Handed Axes
			[1]  = false, -- Two-Handed Axes
			[2]  = false, -- Bows
			[3]  = false, -- Guns
			[4]  = true,  -- One-Handed Maces
			[5]  = false, -- Two-Handed Maces
			[6]  = false, -- Polearms
			[7]  = false, -- One-Handed Swords
			[8]  = false, -- Two-Handed Swords
			[9]  = false, -- Warglaives
			[10] = true,  -- Staves
			[13] = false, -- Fist Weapons
			[15] = true,  -- Daggers
			[18] = false, -- Crossbows
			[19] = true,  -- Wands
		},
		stats = {
			agility 	= false,
			strength 	= false,
			intellect 	= true
		},
	},
	[6] = 				  -- Death Knight
	{
		[4] = 			  -- Armor Class Type
		{
			[1] = false,  -- Cloth Armor 
			[2] = false,  -- Leather Armor
			[3] = false,  -- Mail Armor
			[4] = true,   -- Plate Armor
			[6] = false,  -- Shield Armor
		},
		[2] = 			  -- Weapons Class Type
		{
			[0]  = true,  -- One-Handed Axes
			[1]  = true,  -- Two-Handed Axes
			[2]  = false, -- Bows
			[3]  = false, -- Guns
			[4]  = true,  -- One-Handed Maces
			[5]  = true,  -- Two-Handed Maces
			[6]  = true,  -- Polearms
			[7]  = true,  -- One-Handed Swords
			[8]  = true,  -- Two-Handed Swords
			[9]  = false, -- Warglaives
			[10] = false, -- Staves
			[13] = false, -- Fist Weapons
			[15] = false, -- Daggers
			[18] = false, -- Crossbows
			[19] = false, -- Wands
		},
		stats = {
			agility 	= false,
			strength 	= true,
			intellect 	= false
		},
	},
	[7] = 				  -- Shaman
	{
		[4] = 			  -- Armor Class Type
		{
			[1] = false,  -- Cloth Armor 
			[2] = false,  -- Leather Armor
			[3] = true,   -- Mail Armor
			[4] = false,  -- Plate Armor
			[6] = true,   -- Shield Armor
		},
		[2] = 			  -- Weapons Class Type
		{
			[0]  = true,  -- One-Handed Axes
			[1]  = false, -- Two-Handed Axes
			[2]  = false, -- Bows
			[3]  = false, -- Guns
			[4]  = true,  -- One-Handed Maces
			[5]  = false, -- Two-Handed Maces
			[6]  = false, -- Polearms
			[7]  = false, -- One-Handed Swords
			[8]  = false, -- Two-Handed Swords
			[9]  = false, -- Warglaives
			[10] = true,  -- Staves
			[13] = true,  -- Fist Weapons
			[15] = true,  -- Daggers
			[18] = false, -- Crossbows
			[19] = false, -- Wands
		},
		stats = {
			agility 	= true,
			strength 	= false,
			intellect 	= true
		},
	},
	[8] = 				  -- Mage
	{
		[4] = 			  -- Armor Class Type
		{
			[1] = true,   -- Cloth Armor 
			[2] = false,  -- Leather Armor
			[3] = false,  -- Mail Armor
			[4] = false,  -- Plate Armor
			[6] = false,  -- Shield Armor
		},
		[2] = 			  -- Weapons Class Type
		{
			[0]  = false, -- One-Handed Axes
			[1]  = false, -- Two-Handed Axes
			[2]  = false, -- Bows
			[3]  = false, -- Guns
			[4]  = false, -- One-Handed Maces
			[5]  = false, -- Two-Handed Maces
			[6]  = false, -- Polearms
			[7]  = true,  -- One-Handed Swords
			[8]  = false, -- Two-Handed Swords
			[9]  = false, -- Warglaives
			[10] = true,  -- Staves
			[13] = false, -- Fist Weapons
			[15] = true,  -- Daggers
			[18] = false, -- Crossbows
			[19] = true,  -- Wands
		},
		stats = {
			agility 	= false,
			strength 	= false,
			intellect 	= true
		},
	},
	[9] = 				  -- Warlock
	{
		[4] = 			  -- Armor Class Type
		{
			[1] = true,   -- Cloth Armor 
			[2] = false,  -- Leather Armor
			[3] = false,  -- Mail Armor
			[4] = false,  -- Plate Armor
			[6] = false,  -- Shield Armor
		},
		[2] = 			  -- Weapons Class Type
		{
			[0]  = false, -- One-Handed Axes
			[1]  = false, -- Two-Handed Axes
			[2]  = false, -- Bows
			[3]  = false, -- Guns
			[4]  = false, -- One-Handed Maces
			[5]  = false, -- Two-Handed Maces
			[6]  = false, -- Polearms
			[7]  = true,  -- One-Handed Swords
			[8]  = false, -- Two-Handed Swords
			[9]  = false, -- Warglaives
			[10] = true,  -- Staves
			[13] = false, -- Fist Weapons
			[15] = true,  -- Daggers
			[18] = false, -- Crossbows
			[19] = true,  -- Wands
		},
		stats = {
			agility 	= false,
			strength 	= false,
			intellect 	= true
		},
	},
	[10] = 				  -- Monk
	{
		[4] = 			  -- Armor Class Type
		{
			[1] = false,  -- Cloth Armor 
			[2] = true,   -- Leather Armor
			[3] = false,  -- Mail Armor
			[4] = false,  -- Plate Armor
			[6] = false,  -- Shield Armor
		},
		[2] = 			  -- Weapons Class Type
		{
			[0]  = true,  -- One-Handed Axes
			[1]  = false, -- Two-Handed Axes
			[2]  = false, -- Bows
			[3]  = false, -- Guns
			[4]  = true,  -- One-Handed Maces
			[5]  = false, -- Two-Handed Maces
			[6]  = true,  -- Polearms
			[7]  = true,  -- One-Handed Swords
			[8]  = false, -- Two-Handed Swords
			[9]  = false, -- Warglaives
			[10] = true,  -- Staves
			[13] = true,  -- Fist Weapons
			[15] = false, -- Daggers
			[18] = false, -- Crossbows
			[19] = false, -- Wands
		},
		stats = {
			agility 	= true,
			strength 	= false,
			intellect 	= true
		},
	},
	[11] = 				  -- Druid
	{
		[4] = 			  -- Armor Class Type
		{
			[1] = false,  -- Cloth Armor 
			[2] = true,   -- Leather Armor
			[3] = false,  -- Mail Armor
			[4] = false,  -- Plate Armor
			[6] = false,  -- Shield Armor
		},
		[2] = 			  -- Weapons Class Type
		{
			[0]  = false, -- One-Handed Axes
			[1]  = false, -- Two-Handed Axes
			[2]  = false, -- Bows
			[3]  = false, -- Guns
			[4]  = true,  -- One-Handed Maces
			[5]  = true,  -- Two-Handed Maces
			[6]  = true,  -- Polearms
			[7]  = false, -- One-Handed Swords
			[8]  = false, -- Two-Handed Swords
			[9]  = false, -- Warglaives
			[10] = true,  -- Staves
			[13] = true,  -- Fist Weapons
			[15] = true,  -- Daggers
			[18] = false, -- Crossbows
			[19] = false, -- Wands
		},
		stats = {
			agility 	= true,
			strength 	= true,
			intellect 	= true
		},
	},
	[12] = 				  -- Demon Hunter
	{
		[4] = 			  -- Armor Class Type
		{
			[1] = false,  -- Cloth Armor 
			[2] = true,   -- Leather Armor
			[3] = false,  -- Mail Armor
			[4] = false,  -- Plate Armor
			[6] = false,  -- Shield Armor
		},
		[4] = 			  -- Weapons Class Type
		{
			[0]  = true,  -- One-Handed Axes
			[1]  = false, -- Two-Handed Axes
			[2]  = false, -- Bows
			[3]  = false, -- Guns
			[4]  = false, -- One-Handed Maces
			[5]  = false, -- Two-Handed Maces
			[6]  = false, -- Polearms
			[7]  = true,  -- One-Handed Swords
			[8]  = false, -- Two-Handed Swords
			[9]  = true,  -- Warglaives
			[10] = false, -- Staves
			[13] = true,  -- Fist Weapons
			[15] = false, -- Daggers
			[18] = false, -- Crossbows
			[19] = false, -- Wands
		},
		stats = {
			agility 	= true,
			strength 	= false,
			intellect 	= false
		},
	},
};