-----------------------
----LOCAL VARIABLES----
-----------------------


local ADDON_NAME, namespace = ...;
local functions = namespace.functions;
local variables = namespace.variables;

local llrConfigWindow = CreateFrame("FRAME", "LPTLootRoll_Config", UIParent);
local loadFrame = CreateFrame("FRAME");

--Add config to standard wow interface window.
llrConfigWindow.name = "LPT Loot Roll";
InterfaceOptions_AddCategory(llrConfigWindow);


-----------------------
----LOCAL FUNCTIONS----
-----------------------


--Function for calculating the movement of a slider.
local function sliderCalc(frame, value)
	local halfStep = frame.stepValue / 2;
	value = value + halfStep - (value + halfStep) % frame.stepValue;
	_G[frame:GetName().."Text"]:SetText(value);
	return value;
end

--Function for selecting what mode history is stored, is it temporary or stored between sessions.
local function rollHistoryMode()
	if llrSettings.saveMode then 
		variables.rollHistory = rollHistoryDB.rollHistory;
	else
		table.wipe(rollHistoryDB.rollHistory);
	end
end

--Function for saving data from config window and closing it.
local function saveData()
	if not variables.addonLoaded then return end;
	
	local mainRoll   	= llrConfigWindow.mainSettingFrame.editBox:GetNumber();
	local offRoll   	= llrConfigWindow.offSettingFrame.editBox:GetNumber();
	local mogRoll 	  	= llrConfigWindow.mogSettingFrame.editBox:GetNumber();
	local historyLength = llrConfigWindow.historySettingFrame.editBox:GetNumber();
	
	local leadMode   		= _G["LPTLootRoll_LeadMode"]:GetChecked();
	local scrollMode   		= _G["LPTLootRoll_ScrollMode"]:GetChecked();
	local saveMode   		= _G["LPTLootRoll_SaveMode"]:GetChecked();
	local eventMode     	= _G["LPTLootRoll_EventMode"]:GetChecked();
	local assistMode	 	= _G["LPTLootRoll_AssistMode"]:GetChecked();
	local lootListener 		= _G["LPTLootRoll_LootListener"]:GetChecked();
	local whisperListener 	= _G["LPTLootRoll_WhisperListener"]:GetChecked();
	
	if
		mainRoll 	  == offRoll or 
		mainRoll 	  == mogRoll or 
		offRoll  	  == mogRoll or
		mainRoll 	  == 0 or
		offRoll  	  == 0 or
		mogRoll 	  == 0 or
		historyLength == 0
	then
		print("");
		functions.printOut(
			"LPT Loot Roll: Invalid settings, please try again. Values cannot be the same or set to 0."
		);
		print("");
		return;
	end

	
	llrSettings.leadMode 	  	= leadMode;
	llrSettings.scrollMode 	  	= scrollMode;
	llrSettings.saveMode 	  	= saveMode;
	llrSettings.eventMode	  	= eventMode;
	llrSettings.assistMode 		= assistMode;
	llrSettings.lootListener 	= lootListener;
	llrSettings.whisperListener = whisperListener;

	llrSettings.mainRoll 	  	= mainRoll;
	llrSettings.offRoll 	  	= offRoll;
	llrSettings.mogRoll 	  	= mogRoll;
	llrSettings.historyLength 	= historyLength;
	
	rollHistoryMode();
	functions.controlRollListener();
	functions.historyLengthControl();
	functions.toggleWhisperListener();
	functions.toggleLootListener();
end

--Function for sharing the roll settings of the user.
local function shareSettings()
	local msg = "[LPTLootRoll: " .. variables.playerName .. " - " .. llrSettings.mainRoll .. "-" .. llrSettings.offRoll .. "-" .. llrSettings.mogRoll .. "]";
	if(IsShiftKeyDown()) then
        local editbox = GetCurrentKeyBoardFocus();
        if(editbox) then
        	editbox:Insert(msg);
		end
	end
end

local function setValuesToDefault()
	llrConfigWindow.mainSettingFrame.editBox:SetNumber(100);
	llrConfigWindow.offSettingFrame.editBox:SetNumber(50);
	llrConfigWindow.mogSettingFrame.editBox:SetNumber(25);
	llrConfigWindow.historySettingFrame.editBox:SetNumber(20);
	llrConfigWindow.userScaleSlider:SetValue(1);
	llrConfigWindow.leaderScaleSlider:SetValue(1);
	llrConfigWindow.itemListScaleSlider:SetValue(1);
	
	_G["LPTLootRoll_LeadMode"]:SetChecked(false);
	_G["LPTLootRoll_ScrollMode"]:SetChecked(false);
	_G["LPTLootRoll_SaveMode"]:SetChecked(false);
	_G["LPTLootRoll_EventMode"]:SetChecked(false);
	_G["LPTLootRoll_AssistMode"]:SetChecked(false);
	_G["LPTLootRoll_LootListener"]:SetChecked(false);
	_G["LPTLootRoll_WhisperListener"]:SetChecked(false);
end


-----------------------
-----CONFIG WINDOW-----
-----------------------


--Add logo image.
local t = llrConfigWindow:CreateTexture(nil,"BACKGROUND",nil,-8)
t:SetTexture("Interface\\AddOns\\LPT Loot Roll\\textures\\logo.tga")
t:SetSize(256,256);
t:SetPoint("CENTER", 0, 0);

--Create and set title of the config window.
llrConfigWindow.title = llrConfigWindow:CreateFontString(nil,"OVERLAY");
llrConfigWindow.title:SetFontObject("GameFontNormalLarge");
llrConfigWindow.title:SetPoint("CENTER", llrConfigWindow, "CENTER", 0, -140);
llrConfigWindow.title:SetText("Version: " .. GetAddOnMetadata("LPT Loot Roll", "Version"));

--Create checkboxes.
functions.checkButtonFactory("LPTLootRoll_LeadMode", llrConfigWindow, 12, -30, "Toggle leader roll mode.", "If checked, the leader window will always open for each item.");
functions.checkButtonFactory("LPTLootRoll_ScrollMode", llrConfigWindow, 12, -60, "Reverse roll list direction.", "If checked, the roll lists will have highest values at top and lowest at bottom.");
functions.checkButtonFactory("LPTLootRoll_SaveMode", llrConfigWindow, 12, -90, "Enable session loot memory.", "If checked, roll history will be kept between sessions.");
functions.checkButtonFactory("LPTLootRoll_EventMode", llrConfigWindow, 12, -120, "Enable constant roll listener.", "If checked, the addon will ALWAYS listen for rolls even when the leader window is hidden.");
functions.checkButtonFactory("LPTLootRoll_AssistMode", llrConfigWindow, 12, -150, "Enable assist loot distribution.", "If checked, the addon will allow loot from all raid warnings.");
functions.checkButtonFactory("LPTLootRoll_LootListener", llrConfigWindow, 12, -180, "Enable loot listener.", "If checked, a pop up window will show upon looting a tradeable item. Accepting the pop up will send the item in a whisper to raid leader.");
functions.checkButtonFactory("LPTLootRoll_WhisperListener", llrConfigWindow, 12, -210, "Enable whisper listener", "If checked and the user is raid leader/raid assist with assist loot enabled, will display an itemlist when raiders link them items that the user can link items from.");

--Create frames to hold the roll settings.
llrConfigWindow.mainSettingFrame 	= CreateFrame("Frame", nil, llrConfigWindow);
llrConfigWindow.offSettingFrame 	= CreateFrame("Frame", nil, llrConfigWindow);
llrConfigWindow.mogSettingFrame 	= CreateFrame("Frame", nil, llrConfigWindow);
llrConfigWindow.historySettingFrame = CreateFrame("Frame", nil, llrConfigWindow);

--Create the editBox frames.
llrConfigWindow.mainSettingFrame.editBox 	= CreateFrame("EditBox", nil, llrConfigWindow.mainSettingFrame, "InputBoxTemplate");
llrConfigWindow.offSettingFrame.editBox 	= CreateFrame("EditBox", nil, llrConfigWindow.offSettingFrame, "InputBoxTemplate");
llrConfigWindow.mogSettingFrame.editBox 	= CreateFrame("EditBox", nil, llrConfigWindow.mogSettingFrame, "InputBoxTemplate");
llrConfigWindow.historySettingFrame.editBox = CreateFrame("EditBox", nil, llrConfigWindow.historySettingFrame, "InputBoxTemplate");

--Initialize the roll setting frames.
functions.editBoxFactory(llrConfigWindow.mainSettingFrame, llrConfigWindow.mainSettingFrame.editBox, llrConfigWindow, 12, 70, "Main Roll Max Value:", llrConfigWindow.offSettingFrame);
functions.editBoxFactory(llrConfigWindow.offSettingFrame, llrConfigWindow.offSettingFrame.editBox, llrConfigWindow, 12, 30, "Off Roll Max Value:", llrConfigWindow.mogSettingFrame);
functions.editBoxFactory(llrConfigWindow.mogSettingFrame, llrConfigWindow.mogSettingFrame.editBox, llrConfigWindow, 12, -10, "Mog Roll Max Value:", llrConfigWindow.historySettingFrame);

--Initialize the history length setting editbox.
functions.editBoxFactory(llrConfigWindow.historySettingFrame, llrConfigWindow.historySettingFrame.editBox, llrConfigWindow, 12, -50, "Max length of history:", llrConfigWindow.mainSettingFrame);

--Create the sliders.
llrConfigWindow.userScaleSlider   	= CreateFrame("Slider", "LLR_User_Scaler", llrConfigWindow, "OptionsSliderTemplate");
llrConfigWindow.leaderScaleSlider 	= CreateFrame("Slider", "LLR_Leader_Scaler", llrConfigWindow, "OptionsSliderTemplate");
llrConfigWindow.itemListScaleSlider = CreateFrame("Slider", "LLR_Item_List_Scaler", llrConfigWindow, "OptionsSliderTemplate");

--Use factories to create the sliders, however add script outside of it as to prevent errors involving llrSettings global variable.
functions.sliderFactory(llrConfigWindow, llrConfigWindow.itemListScaleSlider, -12, 170, "Item list window scale");
llrConfigWindow.itemListScaleSlider:SetScript("OnValueChanged", 
	function (self, value)
		llrSettings.itemListScale = sliderCalc(llrConfigWindow.itemListScaleSlider, value);
		functions.updateItemListScale();
	end
);

functions.sliderFactory(llrConfigWindow, llrConfigWindow.userScaleSlider, -12, 100, "User window scale");
llrConfigWindow.userScaleSlider:SetScript("OnValueChanged", 
	function (self, value)
		llrSettings.userScale = sliderCalc(llrConfigWindow.userScaleSlider, value);
		functions.updateUserScale();
	end
);

functions.sliderFactory(llrConfigWindow, llrConfigWindow.leaderScaleSlider, -12, 30, "Leader window scale");
llrConfigWindow.leaderScaleSlider:SetScript("OnValueChanged", 
	function (self, value)
		llrSettings.leaderScale = sliderCalc(llrConfigWindow.leaderScaleSlider, value);
		functions.updateLeaderScale();
	end
);

llrConfigWindow:SetScript("OnShow", 
	function(self) 
		functions.updateConfigWindow();
	end
);

llrConfigWindow.default = function(self) setValuesToDefault() end;
llrConfigWindow.okay = function(self) saveData() end;
llrConfigWindow:Hide();

--Edit history editbox to have a max of two digits.
llrConfigWindow.historySettingFrame.editBox:SetMaxLetters(2);

--Create the share button.
llrConfigWindow.shareButton = CreateFrame("Button", nil, llrConfigWindow, "GameMenuButtonTemplate");
functions.startButton(llrConfigWindow.shareButton, llrConfigWindow, "TOPRIGHT", "TOPRIGHT", -12, -30, "Share roll values", 120, 30, function(self) shareSettings() end);

--Create share button tooltip.
llrConfigWindow.shareButton:SetScript("OnEnter",
function(self)
	GameTooltip:SetOwner(self, "ANCHOR_CURSOR");
	GameTooltip:ClearLines();
	GameTooltip:AddLine(
[[Shift click this button while having a chat window open in order to create
a clickable link that other users can click.

If other users click the link, they will get a pop up asking them if they wish 
to accept your roll settings.
]]
	);
	GameTooltip:Show();
end);

llrConfigWindow.shareButton:SetScript("OnLeave",
function(self)
	GameTooltip:Hide();
	ResetCursor();
end);

--Create the clear data button.
llrConfigWindow.clearData = CreateFrame("Button", nil, llrConfigWindow, "GameMenuButtonTemplate");
functions.startButton(llrConfigWindow.clearData, llrConfigWindow, "TOPRIGHT", "TOPRIGHT", -12, -70, "Clear history", 120, 30, function(self) StaticPopup_Show("LPTLootRoll_ClearHistory") end);

--Create the reset position button.
llrConfigWindow.resetPosition = CreateFrame("Button", nil, llrConfigWindow, "GameMenuButtonTemplate");
functions.startButton(llrConfigWindow.resetPosition, llrConfigWindow, "TOPRIGHT", "TOPRIGHT", -12, -110, "Reset positions", 120, 30, function(self) StaticPopup_Show("LPTLootRoll_ResetPosition") end);


-----------------------
---GLOBAL FUNCTIONS----
-----------------------

--Updating the config windows values.
function functions.updateConfigWindow()
	if not variables.addonLoaded then return end;
	
	llrConfigWindow.mainSettingFrame.editBox:SetNumber(llrSettings.mainRoll);
	llrConfigWindow.offSettingFrame.editBox:SetNumber(llrSettings.offRoll);
	llrConfigWindow.mogSettingFrame.editBox:SetNumber(llrSettings.mogRoll);
	llrConfigWindow.historySettingFrame.editBox:SetNumber(llrSettings.historyLength);

	llrConfigWindow.userScaleSlider:SetValue(llrSettings.userScale);
	llrConfigWindow.leaderScaleSlider:SetValue(llrSettings.leaderScale);
	llrConfigWindow.itemListScaleSlider:SetValue(llrSettings.itemListScale);
	
	_G["LPTLootRoll_LeadMode"]:SetChecked(llrSettings.leadMode);
	_G["LPTLootRoll_ScrollMode"]:SetChecked(llrSettings.scrollMode);
	_G["LPTLootRoll_SaveMode"]:SetChecked(llrSettings.saveMode);
	_G["LPTLootRoll_EventMode"]:SetChecked(llrSettings.eventMode);
	_G["LPTLootRoll_AssistMode"]:SetChecked(llrSettings.assistMode);
	_G["LPTLootRoll_LootListener"]:SetChecked(llrSettings.lootListener);
	_G["LPTLootRoll_WhisperListener"]:SetChecked(llrSettings.whisperListener);
end