-----------------------
----LOCAL VARIABLES----
-----------------------


local ADDON_NAME, namespace = ...
local functions = namespace.functions;
local variables = namespace.variables;

local leaderWindow = CreateFrame("Frame", "LPTLootRoll_Leader", nil, "BasicFrameTemplate");
local loadFrame = CreateFrame("FRAME");
local historyIndex = 1;


-----------------------
----LOCAL FUNCTIONS----
-----------------------


--Function for toggeling of all buttons in a frame.
local function toggleButtons(enabled)
	leaderWindow.rollFrame.mainSpeccButton:SetEnabled(enabled);
	leaderWindow.rollFrame.offSpeccButton:SetEnabled(enabled);
	leaderWindow.rollFrame.mogButton:SetEnabled(enabled);
	leaderWindow.rollFrame.otherButton:SetEnabled(enabled);
end

--Update the names of the lists to reflect their values.
local function updateListNames()
	leaderWindow.mainFrame.title:SetText("Main (Max: " .. llrSettings.mainRoll .. ")");
	leaderWindow.offFrame.title:SetText("Off (Max: " .. llrSettings.offRoll .. ")");
	leaderWindow.mogFrame.title:SetText("Mog (Max: " .. llrSettings.mogRoll .. ")");
end

--Add array to frame function.
local function addArrayToFrame(frame, array)
	for i,v in ipairs(array) do 
		frame:AddMessage("|Hgarrmission:lptlootroll:roller&" .. v[1] .. "|h(" .. v[2] .. ") " .. v[1] .. "|h|r", v[3], v[4], v[5]);
	end
end

--Reverse array function 
local function reverseArray(array)
	local i, j = 1, #array

	while i < j do
		array[i], array[j] = array[j], array[i]
		i = i + 1
		j = j - 1
	end
end

--Function for updating the scroll list message frame.
local function updateMessageframe(frame, array)
	frame:Clear();

	if not array then 
		return;
	end

	local arrLength = #array;

	if arrLength > 0 then
		if arrLength > 7 then 
			frame:GetParent().scrollBar:SetMinMaxValues(0, arrLength-7);
		else
			frame:GetParent().scrollBar:SetMinMaxValues(0,0);
		end

		frame:SetMaxLines(arrLength);
		
		--Sort table based primarly by roll number, secondly by name.
		table.sort(array, 
			function(a,b)
				if a[2] > b[2] then
					return true;
				elseif a[2] < b[2] then
					return false;
				else
					return a[1] < b[1];
				end
			end
		);
	
		--Determine how the frames should be scrolled before displaying them.
		local scrollBarScroll = 0;
		local frameScroll = arrLength-7;

		if not llrSettings.scrollMode then
			reverseArray(array);
			scrollBarScroll = arrLength-7;
			frameScroll = 0;
		end

		addArrayToFrame(frame, array);
		frame:GetParent().scrollBar:SetValue(scrollBarScroll);
		frame:SetScrollOffset(frameScroll);
	end
end

--Function for updating all the message frames with values for item at index.
local function updateAllMessageFrames(index)
	updateMessageframe(leaderWindow.mainFrame.messageFrame, variables.rollHistory[index].mainList);
	updateMessageframe(leaderWindow.offFrame.messageFrame, variables.rollHistory[index].offList);
	updateMessageframe(leaderWindow.mogFrame.messageFrame, variables.rollHistory[index].mogList);
	updateMessageframe(leaderWindow.otherFrame.messageFrame, variables.rollHistory[index].otherList);
end

--Function for adding a roller to an array.
local function addRollerToArray(array, roller)
	--Determine if user has already rolled, and if found then exit function.
	if #array > 0 then
		for _,v in ipairs(array) do 
			if v[1] == roller[1] then
				return;
			end
		end
	end
	
	--Clear the current frame and add the roller to the list.
	tinsert(array, roller);
end

--Function for handling page number display.
local function pageControl()
	leaderWindow.historyButtonFrame.pageNumber:SetText(historyIndex .. " / " .. #variables.rollHistory);
	LPTLootRoll_NextHistoryPage:SetEnabled(#variables.rollHistory > historyIndex);
	LPTLootRoll_PreviousHistoryPage:SetEnabled(historyIndex > 1);
end

--Function for handling history list displays.
local function scrollHistory(bool)
	if #variables.rollHistory > 0 then
	--Increase or decrease history index depending on direction of scroll.
		historyIndex = historyIndex + ((bool and #variables.rollHistory > historyIndex) and 1 or ((not bool and 1 < historyIndex) and -1 or 0));

		pageControl();
		functions.setItemButtonData(leaderWindow.itemButton, variables.rollHistory[historyIndex].item);
		updateAllMessageFrames(historyIndex);
	end
end

--Function for handling mouseover of player names in the roll lists.
local function playerMouseover(_, _, input)
	if historyIndex == 1 then
		local _, _, _, name = input:find("(%b()) (%a+)");

		GameTooltip:SetOwner(UIParent, "ANCHOR_CURSOR"); 
		GameTooltip:ClearLines(); 

		if variables.rollHistory[1].distributed then
			GameTooltip:AddLine("|cFFFFFF00Item has already been distributed.");
		else
			GameTooltip:AddLine("|cFFFFFF00Click to give " .. name .. " the item.");
		end

		GameTooltip:Show();
	end
end

--Function for clearing a frame.
local function clearFrame(frame)
	--Clear list views to prepare for new item.
	frame:Clear();
	
	--Reset scrollbar values. 
	frame:GetParent().scrollBar:SetMinMaxValues(0,0);
	frame:GetParent().scrollBar:SetValue(select(1, frame:GetParent().scrollBar:GetMinMaxValues()));
end


-----------------------
-----EVENT HANDLING----
-----------------------


leaderWindow:SetScript("OnEvent", 
	function(self, event, ...)
		local message = ...;
		local author, roll, rollMin, rollMax = strmatch(message, "(.+) rolls (%d+) %((%d+)-(%d+)%)");
		
		if author then
			--Convert the rolls to numbers.
			roll 	= tonumber(roll);
			rollMin = tonumber(rollMin);
			rollMax = tonumber(rollMax);

			if roll and rollMax and rollMin == 1 then
				local color = functions.getClassColor(author);
				local roller = {author, roll, color.r, color.g, color.b};

				if rollMax == llrSettings.mainRoll then
					addRollerToArray(variables.rollHistory[1].mainList, roller);
				elseif rollMax == llrSettings.offRoll then
					addRollerToArray(variables.rollHistory[1].offList, roller);
				elseif rollMax == llrSettings.mogRoll then
					addRollerToArray(variables.rollHistory[1].mogList, roller);
				elseif rollMax < variables.maxRollValue then
					roller[2] = rollMax;
					addRollerToArray(variables.rollHistory[1].otherList, roller);
				else
					return;
				end

				if historyIndex == 1 then
					updateAllMessageFrames(1);
				end
			end
		end
	end
);


-----------------------
-----LEADER WINDOW-----
-----------------------


leaderWindow:SetToplevel("true")
leaderWindow:SetSize(340,235);
leaderWindow:SetMovable(true);
leaderWindow:EnableMouse(true);
leaderWindow:RegisterForDrag("LeftButton");

leaderWindow:SetScript("OnDragStart", 
	function(self) 
		leaderWindow:StartMoving();
	end
);

leaderWindow:SetScript("OnDragStop", 
	function(self) 
		leaderWindow:StopMovingOrSizing() 
	end
);

leaderWindow:SetScript("OnShow", 
	function(self) 
		functions.updateLeaderScale();
		functions.controlRollListener();
	end
);

leaderWindow:SetScript("OnHide", 
	function(self) 
		if variables.addonLoaded then 
			functions.controlRollListener();
		end;
	end
);

leaderWindow:SetScript("OnMouseWheel", 
	function(self, delta)
		scrollHistory(#variables.rollHistory > 0 and delta ~= 1);
	end
);

leaderWindow.title = leaderWindow:CreateFontString(nil, "OVERLAY");
leaderWindow.title:SetFontObject("GameFontHighLight");
leaderWindow.title:SetPoint("LEFT", leaderWindow.TitleBg, "LEFT", 5, -1);
leaderWindow.title:SetText("LLR Leader");

leaderWindow:SetClampedToScreen(true);
leaderWindow:SetClampRectInsets(0, 0, 0, 0)
leaderWindow:Hide();

--Setup the raid leader item preview.
leaderWindow.itemButton = CreateFrame("Button", nil, leaderWindow, "GameMenuButtonTemplate");
leaderWindow.itemButton:SetPoint("TOPLEFT", leaderWindow, "TOPLEFT", 10, -44);
leaderWindow.itemButton:SetSize(60,60);

leaderWindow.itemButton.itemTexture = leaderWindow.itemButton:CreateTexture(nil,"OVERLAY");
leaderWindow.itemButton.itemTexture:SetPoint("TOPLEFT", 0, 0);
leaderWindow.itemButton.itemTexture:SetPoint("BOTTOMRIGHT", 0, 0);

--Initialize raid leader's roll buttons.
leaderWindow.rollFrame = CreateFrame("Frame","RollButtons", leaderWindow);
leaderWindow.rollFrame:SetSize(60,120);
leaderWindow.rollFrame:SetPoint("LEFT", leaderWindow, "LEFT", 30, 10);

leaderWindow.rollFrame.mainSpeccButton  = CreateFrame("Button", nil, leaderWindow.rollFrame, "GameMenuButtonTemplate");
leaderWindow.rollFrame.offSpeccButton 	= CreateFrame("Button", nil, leaderWindow.rollFrame, "GameMenuButtonTemplate");
leaderWindow.rollFrame.mogButton 		= CreateFrame("Button", nil, leaderWindow.rollFrame, "GameMenuButtonTemplate");
leaderWindow.rollFrame.otherButton 		= CreateFrame("Button", nil, leaderWindow.rollFrame, "GameMenuButtonTemplate");

--Start instance of raid leader buttons.
functions.startButton(leaderWindow.rollFrame.mainSpeccButton, leaderWindow.rollFrame, "CENTER", "TOP", -20, -73, "Main", 60, 30, function(self) RandomRoll(1,llrSettings.mainRoll) toggleButtons(false)  end);
functions.startButton(leaderWindow.rollFrame.offSpeccButton, leaderWindow.rollFrame, "CENTER", "TOP", -20, -103, "Off", 60, 30, function(self) RandomRoll(1,llrSettings.offRoll) toggleButtons(false) end);
functions.startButton(leaderWindow.rollFrame.mogButton, leaderWindow.rollFrame, "CENTER", "TOP", -20, -133, "Mog", 60, 30, function(self) RandomRoll(1,llrSettings.mogRoll) toggleButtons(false) end);
functions.startButton(leaderWindow.rollFrame.otherButton, leaderWindow.rollFrame, "CENTER", "TOP", -20, -163, "Other", 60, 30, function(self) StaticPopup_Show("LPTLootRoll_ManualRoll") end);

--Initalize history buttons
leaderWindow.historyButtonFrame = CreateFrame("Frame", nil, leaderWindow);
leaderWindow.historyButtonFrame:SetSize(20,15);
leaderWindow.historyButtonFrame:SetPoint("TOPRIGHT", leaderWindow, "TOPRIGHT", -25, -10);

leaderWindow.historyButtonFrame.pageNumber = leaderWindow:CreateFontString(nil,"OVERLAY");
leaderWindow.historyButtonFrame.pageNumber:SetFontObject("GameFontHighLight");
leaderWindow.historyButtonFrame.pageNumber:SetPoint("RIGHT", leaderWindow.historyButtonFrame, "RIGHT", -50, 6);

leaderWindow.historyButtonFrame.nextHistoryPage 	= CreateFrame("Button", "LPTLootRoll_NextHistoryPage", leaderWindow.historyButtonFrame, "GameMenuButtonTemplate");
leaderWindow.historyButtonFrame.previousHistoryPage = CreateFrame("Button", "LPTLootRoll_PreviousHistoryPage", leaderWindow.historyButtonFrame, "GameMenuButtonTemplate");

leaderWindow.historyButtonFrame.nextHistoryPage:Disable();
leaderWindow.historyButtonFrame.previousHistoryPage:Disable();

--Start instance of history buttons
functions.startButton(leaderWindow.historyButtonFrame.nextHistoryPage, leaderWindow.historyButtonFrame, "CENTER", "TOP", 0, 0, ">", 20, 20, function(self) scrollHistory(true) end);
functions.startButton(leaderWindow.historyButtonFrame.previousHistoryPage, leaderWindow.historyButtonFrame, "CENTER", "TOP", -22, 0, "<", 20, 20, function(self) scrollHistory(false) end);

--Initialize the frames for the rolled lists.
leaderWindow.mainFrame  = CreateFrame("Frame", "MainList", leaderWindow);
leaderWindow.offFrame   = CreateFrame("Frame", "OffList", leaderWindow);
leaderWindow.mogFrame   = CreateFrame("Frame", "MogList", leaderWindow);
leaderWindow.otherFrame = CreateFrame("Frame", "OtherList", leaderWindow);

--Start the instance of the rolled list frames.
functions.scrollFrameFactory(leaderWindow.mainFrame, leaderWindow, 77, 21, "Main", 120, 130, true);
functions.scrollFrameFactory(leaderWindow.offFrame, leaderWindow, 77, -77, "Off", 120, 130, true);
functions.scrollFrameFactory(leaderWindow.mogFrame, leaderWindow, 207, 21, "Mog", 120, 130, true);
functions.scrollFrameFactory(leaderWindow.otherFrame, leaderWindow, 207, -77, "Other (Max: 9999)", 120, 130, true);

--Enable hyperlinks for all message frames, to allow for clickable names.
leaderWindow.mainFrame.messageFrame:SetHyperlinksEnabled(1);
leaderWindow.mainFrame.messageFrame:SetScript('OnHyperlinkClick', 
	function(self, link, text, button)
		if historyIndex == 1 and not variables.rollHistory[1].distributed then
			ChatFrame_OnHyperlinkShow(self, link, text, button);
		end
	end
);
leaderWindow.mainFrame.messageFrame:SetScript("OnHyperlinkEnter", playerMouseover);
leaderWindow.mainFrame.messageFrame:SetScript("OnHyperlinkLeave", functions.hideGametoolTip);

leaderWindow.offFrame.messageFrame:SetHyperlinksEnabled(1);
leaderWindow.offFrame.messageFrame:SetScript('OnHyperlinkClick', ChatFrame_OnHyperlinkShow);
leaderWindow.offFrame.messageFrame:SetScript("OnHyperlinkEnter", playerMouseover);
leaderWindow.offFrame.messageFrame:SetScript("OnHyperlinkLeave", functions.hideGametoolTip);

leaderWindow.mogFrame.messageFrame:SetHyperlinksEnabled(1);
leaderWindow.mogFrame.messageFrame:SetScript('OnHyperlinkClick', ChatFrame_OnHyperlinkShow);
leaderWindow.mogFrame.messageFrame:SetScript("OnHyperlinkEnter", playerMouseover);
leaderWindow.mogFrame.messageFrame:SetScript("OnHyperlinkLeave", functions.hideGametoolTip);

leaderWindow.otherFrame.messageFrame:SetHyperlinksEnabled(1);
leaderWindow.otherFrame.messageFrame:SetScript('OnHyperlinkClick', ChatFrame_OnHyperlinkShow);
leaderWindow.otherFrame.messageFrame:SetScript("OnHyperlinkEnter", playerMouseover);
leaderWindow.otherFrame.messageFrame:SetScript("OnHyperlinkLeave", functions.hideGametoolTip);


-----------------------
---GLOBAL FUNCTIONS----
-----------------------


--Function for updating scale of the leader window.
function functions.updateLeaderScale()
	leaderWindow:SetScale(variables.screenScale*llrSettings.leaderScale);
end

--Function to disable leader roll buttons.
function functions.disableLeader()
	if leaderWindow:IsShown() then
		toggleButtons(false);
	end
end

--Function for resetting all roll and item history.
function functions.resetRolls()
	clearFrame(leaderWindow.mainFrame.messageFrame);
	clearFrame(leaderWindow.offFrame.messageFrame);
	clearFrame(leaderWindow.mogFrame.messageFrame);
	clearFrame(leaderWindow.otherFrame.messageFrame);
	
	table.wipe(variables.rollHistory);
	
	scrollHistory();
end

--Function for resetting leader window.
function functions.resetLeaderWindow()
	leaderWindow:ClearAllPoints();
	leaderWindow:SetPoint("CENTER", UIParent, "CENTER");
end

--Function to display the leader window.
function functions.displayLeader()
	historyIndex = 1;
	
	scrollHistory();

	--Toggle leaderWindow UI.
	if not leaderWindow:IsShown() then
		leaderWindow:Show();
	end 
	
	updateListNames();
end

--Function for starting the leader window upon a new item drop.
function functions.leaderEvent(isUsable)
	functions.displayLeader();
	
	--If leader can equip, enable buttons, otherwise disable.
	toggleButtons(isUsable);
end

--Function for controlling the event listener for rolls.
function functions.controlRollListener()
	if llrSettings.eventMode or leaderWindow:IsShown() then
		leaderWindow:RegisterEvent("CHAT_MSG_SYSTEM");
	else
		leaderWindow:UnregisterEvent("CHAT_MSG_SYSTEM"); 
	end
end